
/*
Copyright 2007 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function SwitchDialog()
{
	this.sw = null; // switch
	this.st = null; // status
	this.pr = null; // preference
//	this.index = -1; // selected table index

	this.control = null;
	this.listboxSwitches = null;
	this.dialogheaderSwitch = null;
	this.initial = null;
	this.listboxStates = null;
	this.dialogheaderState = null;
	this.descriptionState = null;
	this.tooltip = null;
	this.color = null;
	this.preference = null;
	this.type = null;
	this.value = null;
	this.descriptionPreference = null;
	this.add = null;
	this.update = null;
	this.tree = null;

	this.columns =
	{
		name: [0, 0], // colName: [index, order]
		type: [1, 0],
		value: [2, 0],
		description: [3, 0]
	};

	this.newSwitch = function()
	{
		var sw = new Switch(null, "New Switch", "");
		if(this.renameSwitch(sw))
		{
			var listitem = this.listboxSwitches.appendItem(sw.getName());
			listitem.sw = sw;
			this.listboxSwitches.selectedIndex = this.listboxSwitches.getRowCount() - 1;
		}
	};

	this.renameSwitch = function(sw)
	{
		var refresh = false;
		if(!sw)
		{
			if (this.listboxSwitches.selectedIndex < 0) return;
			sw = this.sw;
			refresh = true;
		}
		var name = prompt("Enter name of switch:", sw.getName());
		if(name == null || name == "") return false;
		sw.setName(name);
		if(refresh)
		{
			var listbox = this.listboxSwitches.getItemAtIndex(this.listboxSwitches.selectedIndex);
			listbox.setAttribute("label", listbox.sw.getName());
			this.refreshSwitches();
		}
		return true;
	};

	this.removeSwitch = function()
	{
		var selIndex = this.listboxSwitches.selectedIndex;
		if(selIndex < 0) return;
		this.listboxSwitches.removeItemAt(selIndex);
//		this.listboxSwitches.selectedIndex = selIndex-1;
		this.refreshSwitches();
	};

	this.newState = function()
	{
		if (this.listboxSwitches.selectedIndex < 0) return;
		var st = new State(null, "New State", "", "", "black");
		if(this.renameState(st))
		{
			this.sw.addState(st);
			var listitem = this.listboxStates.appendItem(st.getName());
			listitem.st = st;
			this.listboxStates.selectedIndex = this.listboxStates.getRowCount() - 1;
		}
	};

	this.renameState = function(st)
	{
		var refresh = false;
		if(!st)
		{
			if (this.listboxStates.selectedIndex < 0) return;
			st = this.st;
			refresh = true;
		}
		var name = prompt("Enter name of state:", st.getName());
		if(name == null || name == "") return false;
		st.setName(name);
		if(refresh)
		{
			var listbox = this.listboxStates.getItemAtIndex(this.listboxStates.selectedIndex);
			listbox.setAttribute("label", listbox.st.getName());
			this.refreshStates(true);
		}
		return true;
	};

	this.removeState = function()
	{
		var selIndex = this.listboxStates.selectedIndex;
		if (selIndex < 0) return;
		this.listboxStates.removeItemAt(selIndex);
		this.sw.removeState(selIndex);
//		this.listboxStates.selectedIndex = selIndex-1;
		this.refreshStates(true);
	};

	this.removeAllStates = function()
	{
		if (this.listboxSwitches.selectedIndex < 0) return;
		this.sw.setStates([]);
		this.refreshStates(true);
	};

	this.removePreference = function()
	{
		if(this.tree.currentIndex < 0) return;
		this.st.removePreference(this.tree.currentIndex);
		this.refreshPreferences(true);
	};

	this.removeAllPreferences = function()
	{
		if (this.listboxStates.selectedIndex < 0) return;
		this.st.setPreferences([]);
		this.refreshPreferences(true);
	};

	this.addPreference = function()
	{
		if (this.pr.isHidden() || this.listboxStates.selectedIndex < 0) return;
		this.st.addPreference(this.pr);
		this.refreshPreferences(true);
	};

	this.updatePreference = function()
	{
		if (this.pr.isHidden() || this.listboxStates.selectedIndex < 0) return;
		this.st.setPreference(this.tree.currentIndex, this.pr);
		this.refreshPreferences(true);
	};

	this.load = function()
	{
		if("arguments" in window && window.arguments.length > 0) this.control = window.arguments[0];
		else this.control = new Control();
		this.listboxSwitches = $("listbox-switches");
		this.dialogheaderSwitch = $("dialogheader-switch");
		this.initial = $("initial");
		this.listboxStates = $("listbox-states");
		this.dialogheaderState = $("dialogheader-state");
		this.descriptionState = $("description-state");
		this.tooltip = $("tooltip");
		this.color = $("color-state");
		this.preference = $("preference");
		this.type = $("type");
		this.value = $("value-state");
		this.descriptionPreference = $("description-preference");
		this.add = $("add-preference");
		this.update = $("update-preference");
		this.tree = $("tree");
		this.listboxSwitches.addEventListener("select", this, false);
		this.listboxStates.addEventListener("select", this, false);
		this.tree.addEventListener("select", this, false);
		this.refreshSwitches(true);
	};

	this.handleEvent = function(event)
	{
		switch(event.target.id)
		{
			case "listbox-switches": this.refreshSwitches(false); break;
			case "listbox-states": this.refreshStates(false); break;
			case "tree": this.refreshPreferences(false);
		}
	};

	this.refreshSwitches = function(load)
	{
try{
		if (load)
		{
			this.listboxSwitches.removeEventListener("select", this, false);
			while(this.listboxSwitches.getRowCount() > 0 && this.listboxSwitches.removeItemAt(0));
			this.listboxSwitches.addEventListener("select", this, false);
			var aSwitch = this.control.getSwitches();
			for(var i=0; i<aSwitch.length; i++)
			{
				var sw = aSwitch[i];
				var listitem = this.listboxSwitches.appendItem(sw.getName());
				listitem.sw = sw;
			}
		}

		if(load || this.listboxSwitches.selectedIndex < 0) this.sw = new Switch(null, "", "");
		else this.sw = this.listboxSwitches.getSelectedItem(0).sw;

		this.dialogheaderSwitch.setAttribute("title", this.sw.getName());
		this.initial.value = this.sw.getInitial();

		this.refreshStates(true);
}catch(e){alert("Switches:"+e);}
	};

	this.refreshStates = function(load)
	{
try{
		if (load)
		{
			this.listboxStates.removeEventListener("select", this, false);
			while(this.listboxStates.getRowCount() > 0 && this.listboxStates.removeItemAt(0));
			this.listboxStates.addEventListener("select", this, false);
			if (this.listboxSwitches.selectedIndex < 0) return;
			var aStates = this.sw.getStates();
			for(var i = 0; i < aStates.length; i++)
			{
				var st = aStates[i];
				var listitem = this.listboxStates.appendItem(st.getName());
				listitem.st = st;
			}
		}

		if(load || this.listboxStates.selectedIndex < 0) this.st = new State(null, "", "", "", "black");
		else this.st = this.listboxStates.getSelectedItem(0).st;

		this.dialogheaderState.setAttribute("title", this.st.getName());
		this.descriptionState.value = this.st.getDescription();
		this.tooltip.value = this.st.getTooltip();
		this.color.value = this.st.getColor();

		this.refreshPreferences(true);
}catch(e){alert("States:"+e);}
	};

	this.refreshPreferences = function(load)
	{
try{
		if (load)
		{
			var aPreferences = this.st ? this.st.getPreferencesArray(): [];
			this.tree.view = new TreeView(aPreferences, this.columns, aPreferences.length);
		}

		if(load || this.tree.currentIndex < 0)
		{
			this.pr = new Preference("", BOOLEAN, "", "");
			this.update.hidden = true;
		}
		else
		{
//			this.index = this.tree.currentIndex;
			this.pr = this.st.getPreference(this.tree.currentIndex).createClone();
			this.update.hidden = false;
		}

		this.preference.value = this.pr.getName();
		this.type.selectedIndex = this.pr.getTypeInt();
		this.value.value = this.pr.getValue();
		this.descriptionPreference.value = this.pr.getDescription();
}catch(e){alert("Preferences:"+e);}
	};

	this.sortColumn = function(column)
	{
		var order = this.columns[column.id][1];
		this.columns[column.id][1] = order == 0 ? 1 : 0;
		this.tableSort(this.sw.getStates(), this.columns[column.id][0], order);
		this.tree.view = new TreeView(this.sw.getStates(), this.columns, this.sw.getStates().length);
	};

	this.tableSort = function(table, col, order)
	{
		var columnSort;
		if(order == 0) columnSort = function(a, b) { return a[col] < b[col] ? -1 : a[col] > b[col] ? 1 : 0; } // ascending
		else columnSort = function(a, b) { return a[col] > b[col] ? -1 : a[col] < b[col] ? 1 : 0; } // descending
		table.sort(columnSort);
	};

	this.accept = function()
	{
		var data = this.generateXml();
		this.control.saveSwitches(data);
		this.control.update(UPDATE_ALL);
	};

	this.cancel = function()
	{
	};

	this.generateXml = function()
	{
		var sResult = "" +
"<?xml version=\"1.0\" encoding=\"" + CHARSET + "\"?>\n" +
COPYRIGHT +
"<prefswitch>\n";
		for(var i=0; i<this.listboxSwitches.getRowCount(); i++)
		{
			var sw = this.listboxSwitches.getItemAtIndex(i).sw;
			sResult += "" +
"	<switch id=\"" + sw.getID() + "\" " +
		"name=\"" + this.control.encode(sw.getName()) + "\" " +
		"initial=\"" + this.control.encode(sw.getInitial()) + "\">\n";
			for(var j=0; j < sw.getStates().length; j++)
			{
				var st = sw.getState(j);
				sResult += "" +
"		<state id=\"" + st.getID() + "\" " +
			"name=\"" + this.control.encode(st.getName()) + "\" " +
			"description=\"" + this.control.encode(st.getDescription()) + "\" " +
			"tooltip=\"" + this.control.encode(st.getTooltip()) + "\" " +
			"color=\"" + this.control.encode(st.getColor()) + "\">\n";
				for(var k=0; k < st.getPreferences().length; k++)
				{
					var tPreference = st.getPreference(k);
					sResult += "" +
"			<preference name=\"" + this.control.encode(tPreference.getName()) + "\" " +
				"type=\"" + tPreference.getType() + "\" " +
				"value=\"" + this.control.encode("" + tPreference.getValue()) + "\" " +
				"description=\"" + this.control.encode(tPreference.getDescription()) + "\"/>\n";
				}
				sResult += "" +
"		</state>\n";
			}
			sResult += "" +
"	</switch>\n";
		}
		sResult += "" +
"</prefswitch>\n";
		return sResult;
	};
}

function TreeView(table, columns, rowcount)
{
	this.table = table;
	this.columns = columns;
	this.rowCount = rowcount;
//	this.cycleHeader = function(column) { sortColumn(column); };
	this.getCellProperties = function(row, col, props) {};
	this.getCellText = function(row, column) { return this.table[row][this.columns[column.id][0]]; };
	this.getColumnProperties = function(colid, col, props) {};
	this.getImageSrc = function(row, col) { return null; };
	this.getLevel = function(row) { return 0; };
	this.getRowProperties = function(row, props) {};
	this.isContainer = function(row) { return false; };
	this.isSeparator = function(row) { return false; };
	this.isSorted = function(row) { return false; };
	this.setTree = function(treebox) { this.treebox=treebox; };
}

function $(id) {return document.getElementById(id);}

var switchDialog = new SwitchDialog();
